# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2010 United States Government as represented by the
# Administrator of the National Aeronautics and Space Administration.
# All Rights Reserved.
#
# Copyright 2010 OpenStack, LLC
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
Installation script for the OpenStack Dashboard development virtualenv.
"""

import os
import subprocess
import sys


ROOT = os.path.dirname(os.path.dirname(os.path.realpath(__file__)))
VENV = os.path.join(ROOT, '.dashboard-venv')
WITH_VENV = os.path.join(ROOT, 'tools', 'with_venv.sh')
PIP_REQUIRES = os.path.join(ROOT, 'tools', 'pip-requires')

def die(message, *args):
  print >>sys.stderr, message % args
  sys.exit(1)


def run_command(cmd, redirect_output=True, check_exit_code=True, cwd=ROOT):
  """
  Runs a command in an out-of-process shell, returning the
  output of that command.  Working directory is ROOT.
  """
  if redirect_output:
    stdout = subprocess.PIPE
  else:
    stdout = None

  proc = subprocess.Popen(cmd, cwd=cwd, stdout=stdout)
  output = proc.communicate()[0]
  if check_exit_code and proc.returncode != 0:
    die('Command "%s" failed.\n%s', ' '.join(cmd), output)
  return output


HAS_EASY_INSTALL = bool(run_command(['which', 'easy_install'], check_exit_code=False).strip())
HAS_VIRTUALENV = bool(run_command(['which', 'virtualenv'], check_exit_code=False).strip())


def check_dependencies():
  """Make sure virtualenv is in the path."""

  if not HAS_VIRTUALENV:
    print 'not found.'
    # Try installing it via easy_install...
    if HAS_EASY_INSTALL:
      print 'Installing virtualenv via easy_install...',
      if not run_command(['which', 'virtualenv']):
        die('ERROR: virtualenv not found.\n\nevelopment requires virtualenv,'
            ' please install it using your favorite package management tool')
      print 'done.'
  print 'done.'


def create_virtualenv(venv=VENV):
  """Creates the virtual environment and installs PIP only into the
  virtual environment
  """
  print 'Creating venv...',
  run_command(['virtualenv', '-q', '--no-site-packages', VENV])
  print 'done.'
  print 'Installing pip in virtualenv...',
  if not run_command([WITH_VENV, 'easy_install', 'pip']).strip():
    die("Failed to install pip.")
  print 'done.'


def install_dependencies(venv=VENV):
  print 'Installing dependencies with pip (this can take a while)...'
  run_command([WITH_VENV, 'pip', 'install', '-E', venv, '-r', PIP_REQUIRES],
              redirect_output=False)

  # Tell the virtual env how to "import dashboard"
  pthfile = os.path.join(venv, "lib", "python2.6", "site-packages", "dashboard.pth")
  f = open(pthfile, 'w')
  f.write("%s\n" % ROOT)


def install_django_nova(path):
    print 'Installing django_nova in development mode...'
    run_command([WITH_VENV, 'python', 'setup.py', 'develop'], cwd=path)

def print_usage():
    usage = """
 OpenStack Dashboard development uses virtualenv to track and manage Python
 dependencies while in development and testing.

 It uses the OpenStack django_nova module.  For more information on how to
 obtian the django_nova module, please refer to the README file.

    usage: python install_venv.py <path to django_nova checkout>
    """
    print usage

def print_summary():
  summary = """
 OpenStack Dashboard development environment setup is complete.

 To activate the virtualenv for the extent of your current shell session you
 can run:

 $ source .dashboard-venv/bin/activate
  """
  print summary


def main(argv):
  if len(argv) != 2:
      print_usage()
      sys.exit(1)
  check_dependencies()
  create_virtualenv()
  install_dependencies()
  install_django_nova(argv[1])
  print_summary()

if __name__ == '__main__':
  main(sys.argv)
